package dev.fyloz.trial.colorrecipesexplorer.core.configuration;

import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.config.MaterialTypeProperties;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;

@Configuration
@Order(Ordered.HIGHEST_PRECEDENCE)
public class InitialDataLoader implements ApplicationListener<ApplicationReadyEvent> {

    private MaterialTypeService materialTypeService;
    private MaterialTypeProperties materialTypeProperties;

    @Autowired
    public InitialDataLoader(MaterialTypeService materialTypeService, MaterialTypeProperties materialTypeProperties) {
        this.materialTypeService = materialTypeService;
        this.materialTypeProperties = materialTypeProperties;
    }

    @Override
    public void onApplicationEvent(ApplicationReadyEvent applicationReadyEvent) {
        for (MaterialType materialType : materialTypeProperties.getDefaults()) {
            if (!materialTypeService.existsByName(materialType.getName())) {
                materialTypeService.save(materialType);
            } else {
                MaterialType found = materialTypeService.getByName(materialType.getName());

                found.setPrefix(materialType.getPrefix());
                found.setUsePercentages(materialType.isUsePercentages());

                materialTypeService.update(found);
            }

            materialTypeService.addDefault(materialType);
        }
    }
}
